@@ -1,5 +1,7 @@
 use crate::formatters::{
-    trivia_formatter::{self, FormatTriviaType},
+    trivia_formatter::{
+        strip_trivia, FormatTriviaType, UpdateLeadingTrivia, UpdateTrailingTrivia, UpdateTrivia,
+    },
     trivia_util, CodeFormatter, Range,
 };
 use full_moon::ast::{
@@ -16,11 +18,7 @@ macro_rules! update_first_token {
     ($enum:ident, $var:ident, $token:expr, $update_method:ident) => {{
         let leading_trivia =
             CodeFormatter::trivia_remove_leading_newlines($token.leading_trivia().collect());
-        let new_token = trivia_formatter::token_reference_add_trivia(
-            $token.to_owned(),
-            FormatTriviaType::Replace(leading_trivia),
-            FormatTriviaType::NoChange,
-        );
+        let new_token = $token.update_leading_trivia(FormatTriviaType::Replace(leading_trivia));
         Stmt::$enum($var.$update_method(new_token))
     }};
 }
@@ -98,15 +96,16 @@ impl CodeFormatter {
             self.format_punctuated(return_node.returns(), &CodeFormatter::format_expression);
 
         let formatted_token = if formatted_returns.is_empty() {
-            trivia_formatter::token_reference_add_trivia(
-                crate::fmt_symbol!(self, return_node.token(), "return"),
+            crate::fmt_symbol!(self, return_node.token(), "return").update_trivia(
                 FormatTriviaType::Append(leading_trivia),
                 FormatTriviaType::Append(trailing_trivia),
             )
         } else {
             // Determine if we need to hang the condition
-            let first_line_str =
-                trivia_formatter::no_comments(return_node.token()) + &formatted_returns.to_string();
+            let first_line_str = strip_trivia(return_node.token()).to_string()
+                + " "
+                + &strip_trivia(&formatted_returns).to_string();
+
             let indent_spacing = (self.indent_level + additional_indent_level.unwrap_or(0))
                 * self.config.indent_width;
             let require_multiline_expression = (indent_spacing
@@ -145,19 +144,13 @@ impl CodeFormatter {
             comments_buf.append(&mut trailing_trivia);
             if let Some(pair) = formatted_returns.pop() {
                 let pair = pair.map(|expr| {
-                    trivia_formatter::expression_add_trailing_trivia(
-                        expr,
-                        FormatTriviaType::Append(comments_buf),
-                    )
+                    expr.update_trailing_trivia(FormatTriviaType::Append(comments_buf))
                 });
                 formatted_returns.push(pair);
             }
 
-            trivia_formatter::token_reference_add_trivia(
-                crate::fmt_symbol!(self, return_node.token(), "return "),
-                FormatTriviaType::Append(leading_trivia),
-                FormatTriviaType::NoChange,
-            )
+            crate::fmt_symbol!(self, return_node.token(), "return ")
+                .update_leading_trivia(FormatTriviaType::Append(leading_trivia))
         };
 
         Return::new()
@@ -170,8 +163,7 @@ impl CodeFormatter {
 
         match last_stmt {
             LastStmt::Break(token) => {
-                LastStmt::Break(trivia_formatter::token_reference_add_trivia(
-                    crate::fmt_symbol!(self, token, "break"),
+                LastStmt::Break(crate::fmt_symbol!(self, token, "break").update_trivia(
                     FormatTriviaType::Append(vec![self.create_indent_trivia(
                         self.get_range_indent_increase(CodeFormatter::get_token_range(token)),
                     )]),
@@ -181,24 +173,24 @@ impl CodeFormatter {
 
             LastStmt::Return(return_node) => LastStmt::Return(self.format_return(return_node)),
             #[cfg(feature = "luau")]
-            LastStmt::Continue(token) => {
-                LastStmt::Continue(trivia_formatter::token_reference_add_trivia(
-                    self.format_symbol(
-                        token,
-                        &TokenReference::new(
-                            vec![],
-                            Token::new(TokenType::Identifier {
-                                identifier: Cow::Owned(String::from("continue")),
-                            }),
-                            vec![],
-                        ),
+            LastStmt::Continue(token) => LastStmt::Continue(
+                self.format_symbol(
+                    token,
+                    &TokenReference::new(
+                        vec![],
+                        Token::new(TokenType::Identifier {
+                            identifier: Cow::Owned(String::from("continue")),
+                        }),
+                        vec![],
                     ),
+                )
+                .update_trivia(
                     FormatTriviaType::Append(vec![self.create_indent_trivia(
                         self.get_range_indent_increase(CodeFormatter::get_token_range(token)),
                     )]),
                     FormatTriviaType::Append(vec![self.create_newline_trivia()]),
-                ))
-            }
+                ),
+            ),
 
             other => panic!("unknown node {:?}", other),
         }
@@ -220,12 +212,7 @@ impl CodeFormatter {
             Prefix::Name(token) => {
                 let leading_trivia =
                     CodeFormatter::trivia_remove_leading_newlines(token.leading_trivia().collect());
-                let new_token = trivia_formatter::token_reference_add_trivia(
-                    token.to_owned(),
-                    FormatTriviaType::Replace(leading_trivia),
-                    FormatTriviaType::NoChange,
-                );
-                Prefix::Name(new_token)
+                Prefix::Name(token.update_leading_trivia(FormatTriviaType::Replace(leading_trivia)))
             }
             Prefix::Expression(expr) => Prefix::Expression(match expr {
                 Expression::Parentheses {
@@ -236,14 +223,10 @@ impl CodeFormatter {
                     let leading_trivia = CodeFormatter::trivia_remove_leading_newlines(
                         start_parens.leading_trivia().collect(),
                     );
-                    let new_token = trivia_formatter::token_reference_add_trivia(
-                        start_parens.to_owned(),
-                        FormatTriviaType::Replace(leading_trivia),
-                        FormatTriviaType::NoChange,
-                    );
                     Expression::Parentheses {
                         contained: full_moon::ast::span::ContainedSpan::new(
-                            new_token,
+                            start_parens
+                                .update_leading_trivia(FormatTriviaType::Replace(leading_trivia)),
                             end_parens.to_owned(),
                         ),
                         expression: Box::new(*expression.to_owned()),
@@ -263,12 +246,7 @@ impl CodeFormatter {
             Var::Name(token) => {
                 let leading_trivia =
                     CodeFormatter::trivia_remove_leading_newlines(token.leading_trivia().collect());
-                let new_token = trivia_formatter::token_reference_add_trivia(
-                    token,
-                    FormatTriviaType::Replace(leading_trivia),
-                    FormatTriviaType::NoChange,
-                );
-                Var::Name(new_token)
+                Var::Name(token.update_leading_trivia(FormatTriviaType::Replace(leading_trivia)))
             }
             Var::Expression(var_expr) => {
                 let prefix = CodeFormatter::prefix_remove_leading_newlines(var_expr.prefix());
@@ -390,22 +368,18 @@ impl CodeFormatter {
             LastStmt::Break(token) => {
                 let leading_trivia =
                     CodeFormatter::trivia_remove_leading_newlines(token.leading_trivia().collect());
-                let new_token = trivia_formatter::token_reference_add_trivia(
-                    token,
-                    FormatTriviaType::Replace(leading_trivia),
-                    FormatTriviaType::NoChange,
-                );
-                LastStmt::Break(new_token)
+                LastStmt::Break(
+                    token.update_leading_trivia(FormatTriviaType::Replace(leading_trivia)),
+                )
             }
             LastStmt::Return(return_node) => {
-                let old_token = return_node.token();
-                let token = trivia_formatter::token_reference_add_trivia(
-                    old_token.to_owned(),
-                    FormatTriviaType::Replace(CodeFormatter::trivia_remove_leading_newlines(
-                        old_token.leading_trivia().collect(),
-                    )),
-                    FormatTriviaType::NoChange,
-                );
+                let token = return_node
+                    .token()
+                    .update_leading_trivia(FormatTriviaType::Replace(
+                        CodeFormatter::trivia_remove_leading_newlines(
+                            return_node.token().leading_trivia().collect(),
+                        ),
+                    ));
 
                 LastStmt::Return(return_node.with_token(token))
             }
@@ -413,12 +387,9 @@ impl CodeFormatter {
             LastStmt::Continue(token) => {
                 let leading_trivia =
                     CodeFormatter::trivia_remove_leading_newlines(token.leading_trivia().collect());
-                let new_token = trivia_formatter::token_reference_add_trivia(
-                    token,
-                    FormatTriviaType::Replace(leading_trivia),
-                    FormatTriviaType::NoChange,
-                );
-                LastStmt::Continue(new_token)
+                LastStmt::Continue(
+                    token.update_leading_trivia(FormatTriviaType::Replace(leading_trivia)),
+                )
             }
             other => panic!("unknown node {:?}", other),
         }
@@ -465,14 +436,13 @@ impl CodeFormatter {
                 true => {
                     let (updated_stmt, trivia) = trivia_util::get_stmt_trailing_trivia(stmt);
                     stmt = updated_stmt;
-                    Some(trivia_formatter::token_reference_add_trivia(
+                    Some(
                         match semi {
                             Some(semi) => crate::fmt_symbol!(self, semi, ";"),
                             None => TokenReference::symbol(";").expect("could not make semicolon"),
-                        },
-                        FormatTriviaType::NoChange,
-                        FormatTriviaType::Append(trivia),
-                    ))
+                        }
+                        .update_trailing_trivia(FormatTriviaType::Append(trivia)),
+                    )
                 }
                 false => match semi {
                     Some(semi) => {
@@ -481,7 +451,7 @@ impl CodeFormatter {
                         let (updated_stmt, trivia) = trivia_util::get_stmt_trailing_trivia(stmt);
                         stmt = updated_stmt;
                         // We will do a hack here, where we insert an empty token, and add all the remaining trivia onto it
-                        Some(trivia_formatter::token_reference_add_trivia(
+                        Some(
                             self.format_symbol(
                                 semi,
                                 &TokenReference::new(
@@ -489,10 +459,9 @@ impl CodeFormatter {
                                     Token::new(TokenType::spaces(0)),
                                     vec![],
                                 ),
-                            ),
-                            FormatTriviaType::NoChange,
-                            FormatTriviaType::Append(trivia),
-                        ))
+                            )
+                            .update_trailing_trivia(FormatTriviaType::Append(trivia)),
+                        )
                     }
                     None => None,
                 },
@@ -504,14 +473,38 @@ impl CodeFormatter {
         // Drop the stmt_iterator as we do not need it anymore and we still need to use `block`
         drop(stmt_iterator);
 
-        let formatted_last_stmt = match block.last_stmt() {
-            Some(last_stmt) => {
+        let formatted_last_stmt = match block.last_stmt_with_semicolon() {
+            Some((last_stmt, semi)) => {
                 let mut last_stmt = self.format_last_stmt(last_stmt);
                 // If this is the first stmt, then remove any leading newlines
                 if !found_first_stmt && self.should_format_node(&last_stmt) {
                     last_stmt = CodeFormatter::last_stmt_remove_leading_newlines(last_stmt);
                 }
-                Some((last_stmt, None))
+                // LastStmt will never need a semicolon
+                // We need to check if we previously had a semicolon, and keep the comments if so
+                let semicolon = match semi {
+                    Some(semi) => {
+                        let (updated_last_stmt, trivia) =
+                            trivia_util::get_last_stmt_trailing_trivia(last_stmt);
+                        last_stmt = updated_last_stmt;
+
+                        // We want to keep any old comments on the semicolon token, otherwise we will lose it
+                        // We will do a hack here, where we replace the semicolon with an empty symbol
+                        let semicolon_token = self
+                            .format_symbol(
+                                semi,
+                                &TokenReference::new(
+                                    vec![],
+                                    Token::new(TokenType::spaces(0)),
+                                    vec![],
+                                ),
+                            )
+                            .update_trailing_trivia(FormatTriviaType::Append(trivia));
+                        Some(semicolon_token)
+                    }
+                    None => None,
+                };
+                Some((last_stmt, semicolon))
             }
             None => None,
         };
