@@ -57,6 +57,9 @@ impl CodeFormatter {
                     // Internal expression is a function call
                     // We could potentially be culling values, so we should not remove parentheses
                     Value::FunctionCall(_) => false,
+                    // Table wrapped around in parentheses
+                    // This could potentially be for a method call on the table, e.g. ({}):foo()
+                    Value::TableConstructor(_) => false,
                     // String literal inside of parentheses
                     // This could be a part of a function call e.g. ("hello"):sub(), so we must leave the parentheses
                     Value::String(_) => false,
