@@ -1,11 +1,14 @@
-use crate::formatters::trivia_formatter::{self, FormatTriviaType};
+use crate::formatters::trivia_formatter::{FormatTriviaType, UpdateTrailingTrivia};
 #[cfg(feature = "luau")]
-use full_moon::ast::types::{IndexedTypeInfo, TypeAssertion, TypeField, TypeFieldKey, TypeInfo};
+use full_moon::ast::span::ContainedSpan;
+#[cfg(feature = "luau")]
+use full_moon::ast::types::{IndexedTypeInfo, TypeDeclaration, TypeInfo};
 use full_moon::{
     ast::{
-        punctuated::Pair, span::ContainedSpan, BinOp, Call, Expression, Field, FunctionArgs, Index,
-        Prefix, Stmt, Suffix, TableConstructor, UnOp, Value, Var,
+        BinOp, Call, Expression, Field, FunctionArgs, Index, LastStmt, Prefix, Stmt, Suffix,
+        TableConstructor, UnOp, Value, Var,
     },
+    node::Node,
     tokenizer::{Token, TokenKind, TokenReference, TokenType},
 };
 
@@ -325,60 +328,272 @@ pub fn get_expression_trailing_comments<'ast>(
         .flatten()
         .collect();
 
-    let new_expression = trivia_formatter::expression_add_trailing_trivia(
-        expression.to_owned(),
-        FormatTriviaType::Replace(vec![]), // TODO: Do we need to keep some trivia?
-    );
+    (
+        expression.update_trailing_trivia(
+            FormatTriviaType::Replace(vec![]), // TODO: Do we need to keep some trivia?
+        ),
+        trailing_comments,
+    )
+}
 
-    (new_expression, trailing_comments)
+/// Macro for retrieving trailing trivia out of a stmt which ends with an `end` token
+macro_rules! end_stmt_trailing_trivia {
+    ($enum:ident, $value:ident) => {{
+        let end_token = $value.end_token();
+        let trailing_trivia = end_token.trailing_trivia().map(|x| x.to_owned()).collect();
+        let new_end_token = end_token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+
+        (
+            Stmt::$enum($value.with_end_token(new_end_token)),
+            trailing_trivia,
+        )
+    }};
+}
+
+#[cfg(feature = "luau")]
+fn get_indexed_type_info_trailing_trivia(
+    type_info: IndexedTypeInfo,
+) -> (IndexedTypeInfo, Vec<Token>) {
+    match type_info {
+        IndexedTypeInfo::Basic(token) => {
+            let trailing_trivia = token.trailing_trivia().map(|x| x.to_owned()).collect();
+            let token = token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            (IndexedTypeInfo::Basic(token), trailing_trivia)
+        }
+        IndexedTypeInfo::Generic {
+            base,
+            arrows,
+            generics,
+        } => {
+            let (start_brace, end_brace) = arrows.tokens();
+            let trailing_trivia = end_brace.trailing_trivia().map(|x| x.to_owned()).collect();
+            let braces = ContainedSpan::new(
+                start_brace.to_owned(),
+                end_brace.update_trailing_trivia(FormatTriviaType::Replace(vec![])),
+            );
+
+            (
+                IndexedTypeInfo::Generic {
+                    base,
+                    arrows: braces,
+                    generics,
+                },
+                trailing_trivia,
+            )
+        }
+        other => panic!("unknown node {:?}", other),
+    }
+}
+
+#[cfg(feature = "luau")]
+fn get_type_info_trailing_trivia(type_info: TypeInfo) -> (TypeInfo, Vec<Token>) {
+    match type_info {
+        TypeInfo::Array { braces, type_info } => {
+            let (start_brace, end_brace) = braces.tokens();
+            let trailing_trivia = end_brace.trailing_trivia().map(|x| x.to_owned()).collect();
+            let braces = ContainedSpan::new(
+                start_brace.to_owned(),
+                end_brace.update_trailing_trivia(FormatTriviaType::Replace(vec![])),
+            );
+
+            (TypeInfo::Array { braces, type_info }, trailing_trivia)
+        }
+        TypeInfo::Basic(token) => {
+            let trailing_trivia = token.trailing_trivia().map(|x| x.to_owned()).collect();
+            let token = token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            (TypeInfo::Basic(token), trailing_trivia)
+        }
+        TypeInfo::Callback {
+            parentheses,
+            arguments,
+            arrow,
+            return_type,
+        } => {
+            let (return_type, trailing_trivia) = get_type_info_trailing_trivia(*return_type);
+            (
+                TypeInfo::Callback {
+                    parentheses,
+                    arguments,
+                    arrow,
+                    return_type: Box::new(return_type),
+                },
+                trailing_trivia,
+            )
+        }
+        TypeInfo::Generic {
+            base,
+            arrows,
+            generics,
+        } => {
+            let (start_brace, end_brace) = arrows.tokens();
+            let trailing_trivia = end_brace.trailing_trivia().map(|x| x.to_owned()).collect();
+            let braces = ContainedSpan::new(
+                start_brace.to_owned(),
+                end_brace.update_trailing_trivia(FormatTriviaType::Replace(vec![])),
+            );
+
+            (
+                TypeInfo::Generic {
+                    base,
+                    arrows: braces,
+                    generics,
+                },
+                trailing_trivia,
+            )
+        }
+        TypeInfo::Intersection {
+            left,
+            ampersand,
+            right,
+        } => {
+            let (right, trailing_trivia) = get_type_info_trailing_trivia(*right);
+            (
+                TypeInfo::Intersection {
+                    left,
+                    ampersand,
+                    right: Box::new(right),
+                },
+                trailing_trivia,
+            )
+        }
+        TypeInfo::Module {
+            module,
+            punctuation,
+            type_info,
+        } => {
+            let (type_info, trailing_trivia) = get_indexed_type_info_trailing_trivia(*type_info);
+            (
+                TypeInfo::Module {
+                    module,
+                    punctuation,
+                    type_info: Box::new(type_info),
+                },
+                trailing_trivia,
+            )
+        }
+        TypeInfo::Optional {
+            base,
+            question_mark,
+        } => {
+            let trailing_trivia = question_mark
+                .trailing_trivia()
+                .map(|x| x.to_owned())
+                .collect();
+            let question_mark =
+                question_mark.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            (
+                TypeInfo::Optional {
+                    base,
+                    question_mark,
+                },
+                trailing_trivia,
+            )
+        }
+        TypeInfo::Table { braces, fields } => {
+            let (start_brace, end_brace) = braces.tokens();
+            let trailing_trivia = end_brace.trailing_trivia().map(|x| x.to_owned()).collect();
+            let braces = ContainedSpan::new(
+                start_brace.to_owned(),
+                end_brace.update_trailing_trivia(FormatTriviaType::Replace(vec![])),
+            );
+
+            (TypeInfo::Table { braces, fields }, trailing_trivia)
+        }
+        TypeInfo::Typeof {
+            typeof_token,
+            parentheses,
+            inner,
+        } => {
+            let (start_brace, end_brace) = parentheses.tokens();
+            let trailing_trivia = end_brace.trailing_trivia().map(|x| x.to_owned()).collect();
+            let braces = ContainedSpan::new(
+                start_brace.to_owned(),
+                end_brace.update_trailing_trivia(FormatTriviaType::Replace(vec![])),
+            );
+
+            (
+                TypeInfo::Typeof {
+                    typeof_token,
+                    parentheses: braces,
+                    inner,
+                },
+                trailing_trivia,
+            )
+        }
+        TypeInfo::Tuple { parentheses, types } => {
+            let (start_brace, end_brace) = parentheses.tokens();
+            let trailing_trivia = end_brace.trailing_trivia().map(|x| x.to_owned()).collect();
+            let braces = ContainedSpan::new(
+                start_brace.to_owned(),
+                end_brace.update_trailing_trivia(FormatTriviaType::Replace(vec![])),
+            );
+
+            (
+                TypeInfo::Tuple {
+                    types,
+                    parentheses: braces,
+                },
+                trailing_trivia,
+            )
+        }
+        TypeInfo::Union { left, pipe, right } => {
+            let (right, trailing_trivia) = get_type_info_trailing_trivia(*right);
+            (
+                TypeInfo::Union {
+                    left,
+                    pipe,
+                    right: Box::new(right),
+                },
+                trailing_trivia,
+            )
+        }
+        other => panic!("unknown node {:?}", other),
+    }
+}
+
+#[cfg(feature = "luau")]
+fn get_type_declaration_trailing_trivia(
+    type_declaration: TypeDeclaration,
+) -> (TypeDeclaration, Vec<Token>) {
+    let (type_definition, trailing_trivia) =
+        get_type_info_trailing_trivia(type_declaration.type_definition().to_owned());
+    (
+        type_declaration.with_type_definition(type_definition),
+        trailing_trivia,
+    )
 }
 
 pub fn get_stmt_trailing_trivia(stmt: Stmt) -> (Stmt, Vec<Token>) {
-    let mut trailing_trivia = Vec::new();
-    let updated_stmt = match stmt {
+    let (updated_stmt, trailing_trivia) = match stmt {
         Stmt::Assignment(assignment) => {
             let mut formatted_expression_list = assignment.expressions().to_owned();
+            let mut trailing_trivia = Vec::new();
             if let Some(last_pair) = formatted_expression_list.pop() {
-                match last_pair {
-                    Pair::End(value) => {
-                        trailing_trivia = get_expression_trailing_trivia(&value);
-                        let expression = trivia_formatter::expression_add_trailing_trivia(
-                            value,
-                            FormatTriviaType::Replace(vec![]),
-                        );
-                        formatted_expression_list.push(Pair::End(expression));
-                    }
-                    Pair::Punctuated(_, _) => {
-                        panic!("we got a punctuated as the last sequence in expression")
-                    }
-                }
+                let pair = last_pair.map(|value| {
+                    trailing_trivia = get_expression_trailing_trivia(&value);
+                    value.update_trailing_trivia(FormatTriviaType::Replace(vec![]))
+                });
+                formatted_expression_list.push(pair);
             }
 
-            Stmt::Assignment(assignment.with_expressions(formatted_expression_list))
+            (
+                Stmt::Assignment(assignment.with_expressions(formatted_expression_list)),
+                trailing_trivia,
+            )
         }
 
         Stmt::LocalAssignment(local_assignment) => {
+            let mut trailing_trivia = Vec::new();
             let new_assignment = if local_assignment.expressions().is_empty() {
                 // Unassigned local variable
                 let mut formatted_name_list = local_assignment.names().to_owned();
-
                 // Retrieve last item and take its trailing comments
                 if let Some(last_pair) = formatted_name_list.pop() {
-                    match last_pair {
-                        Pair::End(value) => {
-                            trailing_trivia =
-                                value.trailing_trivia().map(|x| x.to_owned()).collect();
-                            let value = trivia_formatter::token_reference_add_trivia(
-                                value,
-                                FormatTriviaType::NoChange,
-                                FormatTriviaType::Replace(vec![]),
-                            );
-                            formatted_name_list.push(Pair::End(value));
-                        }
-                        Pair::Punctuated(_, _) => {
-                            panic!("punctuated sequence not ended with a Pair::End")
-                        }
-                    }
+                    let pair = last_pair.map(|value| {
+                        trailing_trivia = value.trailing_trivia().map(|x| x.to_owned()).collect();
+                        value.update_trailing_trivia(FormatTriviaType::Replace(vec![]))
+                    });
+                    formatted_name_list.push(pair);
                 }
                 local_assignment.with_names(formatted_name_list)
             } else {
@@ -388,54 +603,193 @@ pub fn get_stmt_trailing_trivia(stmt: Stmt) -> (Stmt, Vec<Token>) {
 
                 // Retrieve last item and remove trailing trivia
                 if let Some(last_pair) = formatted_expression_list.pop() {
-                    match last_pair {
-                        Pair::End(value) => {
-                            trailing_trivia = get_expression_trailing_trivia(&value);
-                            let expression = trivia_formatter::expression_add_trailing_trivia(
-                                value,
-                                FormatTriviaType::Replace(vec![]),
-                            );
-                            formatted_expression_list.push(Pair::End(expression));
-                        }
-                        Pair::Punctuated(_, _) => {
-                            panic!("got a last pair which was punctuated")
-                        }
-                    }
+                    let pair = last_pair.map(|value| {
+                        trailing_trivia = get_expression_trailing_trivia(&value);
+                        value.update_trailing_trivia(FormatTriviaType::Replace(vec![]))
+                    });
+                    formatted_expression_list.push(pair);
                 }
 
                 local_assignment.with_expressions(formatted_expression_list)
             };
 
-            Stmt::LocalAssignment(new_assignment)
+            (Stmt::LocalAssignment(new_assignment), trailing_trivia)
         }
 
         Stmt::FunctionCall(function_call) => {
             let last_suffix = function_call.suffixes().last();
-            trailing_trivia = match last_suffix {
+            let trailing_trivia = match last_suffix {
                 Some(suffix) => suffix_trailing_trivia(suffix),
                 None => Vec::new(),
             };
 
-            Stmt::FunctionCall(trivia_formatter::function_call_add_trailing_trivia(
-                function_call,
-                FormatTriviaType::Replace(vec![]),
-            ))
+            (
+                Stmt::FunctionCall(
+                    function_call.update_trailing_trivia(FormatTriviaType::Replace(vec![])),
+                ),
+                trailing_trivia,
+            )
         }
         Stmt::Repeat(repeat_block) => {
-            trailing_trivia = get_expression_trailing_trivia(repeat_block.until());
-            let until_expr = trivia_formatter::expression_add_trailing_trivia(
-                repeat_block.until().to_owned(),
-                FormatTriviaType::Replace(vec![]),
-            );
+            let trailing_trivia = get_expression_trailing_trivia(repeat_block.until());
+            let until_expr = repeat_block
+                .until()
+                .update_trailing_trivia(FormatTriviaType::Replace(vec![]));
 
-            Stmt::Repeat(repeat_block.with_until(until_expr))
+            (
+                Stmt::Repeat(repeat_block.with_until(until_expr)),
+                trailing_trivia,
+            )
         }
-        _ => panic!("stmt trailing comments not implemented"),
+
+        Stmt::Do(stmt) => {
+            end_stmt_trailing_trivia!(Do, stmt)
+        }
+        Stmt::GenericFor(stmt) => {
+            end_stmt_trailing_trivia!(GenericFor, stmt)
+        }
+        Stmt::If(stmt) => {
+            end_stmt_trailing_trivia!(If, stmt)
+        }
+        Stmt::FunctionDeclaration(stmt) => {
+            let end_token = stmt.body().end_token();
+            let trailing_trivia = end_token.trailing_trivia().map(|x| x.to_owned()).collect();
+            let new_end_token = end_token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+
+            let body = stmt.body().to_owned().with_end_token(new_end_token);
+            (
+                Stmt::FunctionDeclaration(stmt.with_body(body)),
+                trailing_trivia,
+            )
+        }
+        Stmt::LocalFunction(stmt) => {
+            let end_token = stmt.body().end_token();
+            let trailing_trivia = end_token.trailing_trivia().map(|x| x.to_owned()).collect();
+            let new_end_token = end_token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+
+            let body = stmt.body().to_owned().with_end_token(new_end_token);
+            (
+                Stmt::LocalFunction(stmt.with_func_body(body)),
+                trailing_trivia,
+            )
+        }
+        Stmt::NumericFor(stmt) => {
+            end_stmt_trailing_trivia!(NumericFor, stmt)
+        }
+        Stmt::While(stmt) => {
+            end_stmt_trailing_trivia!(While, stmt)
+        }
+
+        #[cfg(feature = "luau")]
+        Stmt::CompoundAssignment(stmt) => {
+            let trailing_trivia = get_expression_trailing_trivia(stmt.rhs());
+            let expr = stmt
+                .rhs()
+                .update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            (
+                Stmt::CompoundAssignment(stmt.with_rhs(expr)),
+                trailing_trivia,
+            )
+        }
+        #[cfg(feature = "luau")]
+        Stmt::ExportedTypeDeclaration(stmt) => {
+            let (type_declaration, trailing_trivia) =
+                get_type_declaration_trailing_trivia(stmt.type_declaration().to_owned());
+            (
+                Stmt::ExportedTypeDeclaration(stmt.with_type_declaration(type_declaration)),
+                trailing_trivia,
+            )
+        }
+        #[cfg(feature = "luau")]
+        Stmt::TypeDeclaration(stmt) => {
+            let (type_declaration, trailing_trivia) = get_type_declaration_trailing_trivia(stmt);
+            (Stmt::TypeDeclaration(type_declaration), trailing_trivia)
+        }
+        #[cfg(feature = "lua52")]
+        Stmt::Goto(stmt) => {
+            let trailing_trivia = stmt
+                .label_name()
+                .trailing_trivia()
+                .map(|x| x.to_owned())
+                .collect();
+            let label_name = stmt
+                .label_name()
+                .update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            (
+                Stmt::Goto(stmt.with_label_name(label_name)),
+                trailing_trivia,
+            )
+        }
+        #[cfg(feature = "lua52")]
+        Stmt::Label(stmt) => {
+            let trailing_trivia = stmt
+                .right_colons()
+                .trailing_trivia()
+                .map(|x| x.to_owned())
+                .collect();
+            let right_colons = stmt
+                .right_colons()
+                .update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            (
+                Stmt::Label(stmt.with_right_colons(right_colons)),
+                trailing_trivia,
+            )
+        }
+
+        other => panic!("unknown node {:?}", other),
     };
 
     (updated_stmt, trailing_trivia)
 }
 
+pub fn get_last_stmt_trailing_trivia(last_stmt: LastStmt) -> (LastStmt, Vec<Token>) {
+    match last_stmt {
+        LastStmt::Return(ret) => {
+            let mut return_token = ret.token().to_owned();
+            let mut formatted_expression_list = ret.returns().to_owned();
+            let mut trailing_trivia = Vec::new();
+
+            // Retrieve last item and remove trailing trivia
+            if let Some(last_pair) = formatted_expression_list.pop() {
+                let pair = last_pair.map(|value| {
+                    trailing_trivia = get_expression_trailing_trivia(&value);
+                    value.update_trailing_trivia(FormatTriviaType::Replace(vec![]))
+                });
+                formatted_expression_list.push(pair);
+            } else {
+                trailing_trivia = return_token
+                    .trailing_trivia()
+                    .map(|x| x.to_owned())
+                    .collect();
+                return_token =
+                    return_token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            }
+
+            (
+                LastStmt::Return(
+                    ret.with_token(return_token)
+                        .with_returns(formatted_expression_list),
+                ),
+                trailing_trivia,
+            )
+        }
+        LastStmt::Break(token) => {
+            let trailing_trivia = token.trailing_trivia().map(|x| x.to_owned()).collect();
+            let token = token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+
+            (LastStmt::Break(token), trailing_trivia)
+        }
+        #[cfg(feature = "luau")]
+        LastStmt::Continue(token) => {
+            let trailing_trivia = token.trailing_trivia().map(|x| x.to_owned()).collect();
+            let token = token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+
+            (LastStmt::Continue(token), trailing_trivia)
+        }
+        other => panic!("unknown node {:?}", other),
+    }
+}
+
 pub fn token_trivia_contains_comments<'ast>(
     trivia: impl Iterator<Item = &'ast Token<'ast>>,
 ) -> bool {
@@ -454,396 +808,43 @@ pub fn token_contains_comments(token_ref: &TokenReference) -> bool {
         || token_trivia_contains_comments(token_ref.trailing_trivia())
 }
 
+pub fn contains_comments<'ast>(node: impl Node<'ast>) -> bool {
+    node.tokens().into_iter().any(token_contains_comments)
+}
+
+/// Checks whether any [`Field`] within a [`TableConstructor`] contains comments, without checking the braces
 pub fn table_fields_contains_comments(table_constructor: &TableConstructor) -> bool {
     table_constructor.fields().pairs().any(|field| {
-        let mut contains_comments = match field.value() {
+        let comments = match field.value() {
             Field::ExpressionKey {
                 brackets,
                 key,
                 equal,
                 value,
             } => {
-                let (start, end) = brackets.tokens();
-                token_contains_comments(start)
-                    || token_contains_comments(end)
-                    || token_contains_comments(equal)
-                    || expression_contains_comments(value)
-                    || expression_contains_comments(key)
+                contains_comments(brackets)
+                    || contains_comments(key)
+                    || contains_comments(equal)
+                    || contains_comments(value)
             }
             Field::NameKey { key, equal, value } => {
-                token_contains_comments(equal)
-                    || token_contains_comments(key)
-                    || expression_contains_comments(value)
+                contains_comments(key) || contains_comments(equal) || contains_comments(value)
             }
-            Field::NoKey(expression) => expression_contains_comments(expression),
+            Field::NoKey(expression) => contains_comments(expression),
             other => panic!("unknown node {:?}", other),
         };
 
-        if let Some(punctuation) = field.punctuation() {
-            if token_contains_comments(punctuation) {
-                contains_comments = true;
-            }
-        }
-
-        contains_comments
+        comments || field.punctuation().map_or(false, contains_comments)
     })
 }
 
-fn table_constructor_contains_comments(table_constructor: &TableConstructor) -> bool {
-    let (start, end) = table_constructor.braces().tokens();
-    token_contains_comments(start)
-        || token_contains_comments(end)
-        || table_fields_contains_comments(table_constructor)
-}
-
-fn function_args_contains_comments(function_args: &FunctionArgs) -> bool {
-    match function_args {
-        FunctionArgs::Parentheses {
-            parentheses,
-            arguments,
-        } => {
-            let (start, end) = parentheses.tokens();
-            if token_contains_comments(start) || token_contains_comments(end) {
-                true
-            } else {
-                let mut contains_comments = false;
-                for argument in arguments.pairs() {
-                    contains_comments = expression_contains_comments(argument.value());
-                    if let Some(punctuation) = argument.punctuation() {
-                        if token_contains_comments(punctuation) {
-                            contains_comments = true;
-                        }
-                    }
-                    if contains_comments {
-                        break;
-                    }
-                }
-                contains_comments
-            }
-        }
-        FunctionArgs::String(token) => token_contains_comments(token),
-        FunctionArgs::TableConstructor(table_constructor) => {
-            table_constructor_contains_comments(table_constructor)
-        }
-        other => panic!("unknown node {:?}", other),
-    }
-}
-
-fn suffix_contains_comments(suffix: &Suffix) -> bool {
-    match suffix {
-        Suffix::Call(call) => match call {
-            Call::AnonymousCall(function_args) => function_args_contains_comments(function_args),
-            Call::MethodCall(method_call) => {
-                token_contains_comments(method_call.name())
-                    || token_contains_comments(method_call.colon_token())
-                    || function_args_contains_comments(method_call.args())
-            }
-            other => panic!("unknown node {:?}", other),
-        },
-        Suffix::Index(index) => match index {
-            Index::Brackets {
-                brackets,
-                expression,
-            } => {
-                let (start, end) = brackets.tokens();
-                token_contains_comments(start)
-                    || token_contains_comments(end)
-                    || expression_contains_comments(expression)
-            }
-            Index::Dot { dot, name } => {
-                token_contains_comments(dot) || token_contains_comments(name)
-            }
-            other => panic!("unknown node {:?}", other),
-        },
-        other => panic!("unknown node {:?}", other),
-    }
-}
-
-fn contained_span_contains_comments(contained_span: &ContainedSpan) -> bool {
-    let (start, end) = contained_span.tokens();
-    token_contains_comments(start) || token_contains_comments(end)
-}
-
-#[cfg(feature = "luau")]
-fn type_info_contains_comments(type_info: &TypeInfo) -> bool {
-    match type_info {
-        TypeInfo::Array { braces, type_info } => {
-            contained_span_contains_comments(braces) || type_info_contains_comments(type_info)
-        }
-        TypeInfo::Basic(token) => token_contains_comments(token),
-        TypeInfo::Callback {
-            parentheses,
-            arguments,
-            arrow,
-            return_type,
-        } => {
-            contained_span_contains_comments(parentheses)
-                || token_contains_comments(arrow)
-                || type_info_contains_comments(return_type)
-                || arguments.pairs().any(|pair| {
-                    type_info_contains_comments(pair.value())
-                        || pair
-                            .punctuation()
-                            .map_or(false, |punc| token_contains_comments(punc))
-                })
-        }
-        TypeInfo::Generic {
-            base,
-            arrows,
-            generics,
-        } => {
-            token_contains_comments(base)
-                || contained_span_contains_comments(arrows)
-                || generics.pairs().any(|pair| {
-                    type_info_contains_comments(pair.value())
-                        || pair
-                            .punctuation()
-                            .map_or(false, |punc| token_contains_comments(punc))
-                })
-        }
-        TypeInfo::Intersection {
-            left,
-            ampersand,
-            right,
-        } => {
-            type_info_contains_comments(left)
-                || token_contains_comments(ampersand)
-                || type_info_contains_comments(right)
-        }
-        TypeInfo::Module {
-            module,
-            punctuation,
-            type_info,
-        } => {
-            token_contains_comments(module)
-                || token_contains_comments(punctuation)
-                || indexed_type_info_contains_comments(type_info)
-        }
-        TypeInfo::Optional {
-            base,
-            question_mark,
-        } => type_info_contains_comments(base) || token_contains_comments(question_mark),
-        TypeInfo::Table { braces, fields } => {
-            contained_span_contains_comments(braces)
-                || fields.pairs().any(|pair| {
-                    type_field_contains_comments(pair.value())
-                        || pair
-                            .punctuation()
-                            .map_or(false, |punc| token_contains_comments(punc))
-                })
-        }
-        TypeInfo::Typeof {
-            typeof_token,
-            parentheses,
-            inner,
-        } => {
-            token_contains_comments(typeof_token)
-                || contained_span_contains_comments(parentheses)
-                || expression_contains_comments(inner)
-        }
-        TypeInfo::Tuple { parentheses, types } => {
-            contained_span_contains_comments(parentheses)
-                || types.pairs().any(|pair| {
-                    type_info_contains_comments(pair.value())
-                        || pair
-                            .punctuation()
-                            .map_or(false, |punc| token_contains_comments(punc))
-                })
-        }
-        TypeInfo::Union { left, pipe, right } => {
-            type_info_contains_comments(left)
-                || token_contains_comments(pipe)
-                || type_info_contains_comments(right)
-        }
-        other => panic!("unknown node {:?}", other),
-    }
-}
-
-#[cfg(feature = "luau")]
-fn indexed_type_info_contains_comments(type_info: &IndexedTypeInfo) -> bool {
-    match type_info {
-        IndexedTypeInfo::Basic(token) => token_contains_comments(token),
-        IndexedTypeInfo::Generic {
-            base,
-            arrows,
-            generics,
-        } => {
-            token_contains_comments(base)
-                || contained_span_contains_comments(arrows)
-                || generics.pairs().any(|pair| {
-                    type_info_contains_comments(pair.value())
-                        || pair
-                            .punctuation()
-                            .map_or(false, |punc| token_contains_comments(punc))
-                })
-        }
-        other => panic!("unknown node {:?}", other),
-    }
-}
-
-#[cfg(feature = "luau")]
-fn type_field_contains_comments(type_field: &TypeField) -> bool {
-    type_field_key_contains_comments(type_field.key())
-        || token_contains_comments(type_field.colon_token())
-        || type_info_contains_comments(type_field.value())
-}
-
-#[cfg(feature = "luau")]
-fn type_field_key_contains_comments(type_field_key: &TypeFieldKey) -> bool {
-    match type_field_key {
-        TypeFieldKey::Name(token) => token_contains_comments(token),
-        TypeFieldKey::IndexSignature { brackets, inner } => {
-            contained_span_contains_comments(brackets) || type_info_contains_comments(inner)
-        }
-        other => panic!("unknown node {:?}", other),
-    }
-}
-
-#[cfg(feature = "luau")]
-fn type_assertion_contains_comments(type_assertion: &TypeAssertion) -> bool {
-    token_contains_comments(type_assertion.assertion_op())
-        || type_info_contains_comments(type_assertion.cast_to())
-}
-
-fn value_contains_comments(value: &Value) -> bool {
-    match value {
-        Value::Function((token, body)) => {
-            if token_contains_comments(token) {
-                true
-            } else {
-                contained_span_contains_comments(body.parameters_parentheses())
-                // TODO: Do we need to do any more?
-            }
-        }
-        Value::FunctionCall(function_call) => {
-            let contained = match function_call.prefix() {
-                Prefix::Name(token) => token_contains_comments(token),
-                Prefix::Expression(expression) => expression_contains_comments(expression),
-                other => panic!("unknown node {:?}", other),
-            };
-
-            if contained {
-                true
-            } else {
-                let mut contained_comments = false;
-                for suffix in function_call.suffixes() {
-                    contained_comments = suffix_contains_comments(suffix);
-                    if contained_comments {
-                        break;
-                    }
-                }
-                contained_comments
-            }
-        }
-        Value::TableConstructor(table_constructor) => {
-            table_constructor_contains_comments(table_constructor)
-        }
-        Value::Number(token) => token_contains_comments(token),
-        Value::ParenthesesExpression(expression) => expression_contains_comments(expression),
-        Value::String(token) => token_contains_comments(token),
-        Value::Symbol(token) => token_contains_comments(token),
-        Value::Var(var) => match var {
-            Var::Name(token) => token_contains_comments(token),
-            Var::Expression(var_expr) => {
-                let contained = match var_expr.prefix() {
-                    Prefix::Name(token) => token_contains_comments(token),
-                    Prefix::Expression(expression) => expression_contains_comments(expression),
-                    other => panic!("unknown node {:?}", other),
-                };
-
-                if contained {
-                    true
-                } else {
-                    let mut contained_comments = false;
-                    for suffix in var_expr.suffixes() {
-                        contained_comments = suffix_contains_comments(suffix);
-                        if contained_comments {
-                            break;
-                        }
-                    }
-                    contained_comments
-                }
-            }
-            other => panic!("unknown node {:?}", other),
-        },
-        other => panic!("unknown node {:?}", other),
-    }
-}
-
-fn binop_contains_comments(binop: &BinOp) -> bool {
-    match binop {
-        BinOp::And(t)
-        | BinOp::Caret(t)
-        | BinOp::GreaterThan(t)
-        | BinOp::GreaterThanEqual(t)
-        | BinOp::LessThan(t)
-        | BinOp::LessThanEqual(t)
-        | BinOp::Minus(t)
-        | BinOp::Or(t)
-        | BinOp::Percent(t)
-        | BinOp::Plus(t)
-        | BinOp::Slash(t)
-        | BinOp::Star(t)
-        | BinOp::TildeEqual(t)
-        | BinOp::TwoDots(t)
-        | BinOp::TwoEqual(t) => token_contains_comments(t),
-        other => panic!("unknown node {:?}", other),
-    }
-}
-
-// Check whether any comments are present within an Expression
-pub fn expression_contains_comments(expression: &Expression) -> bool {
-    match expression {
-        Expression::Parentheses {
-            contained,
-            expression,
-        } => {
-            contained_span_contains_comments(contained) || expression_contains_comments(expression)
-        }
-        Expression::UnaryOperator { unop, expression } => {
-            match unop {
-                UnOp::Minus(token) | UnOp::Not(token) | UnOp::Hash(token) => {
-                    if token_contains_comments(token) {
-                        return true;
-                    }
-                }
-                other => panic!("unknown node {:?}", other),
-            }
-
-            expression_contains_comments(expression)
-        }
-        Expression::BinaryOperator { lhs, binop, rhs } => {
-            binop_contains_comments(binop)
-                || expression_contains_comments(lhs)
-                || expression_contains_comments(rhs)
-        }
-        Expression::Value {
-            value,
-            #[cfg(feature = "luau")]
-            type_assertion,
-        } => {
-            #[cfg(feature = "luau")]
-            {
-                return value_contains_comments(value)
-                    || type_assertion
-                        .as_ref()
-                        .map_or(false, |x| type_assertion_contains_comments(x));
-            }
-
-            #[cfg(not(feature = "luau"))]
-            value_contains_comments(value)
-        }
-        other => panic!("unknown node {:?}", other),
-    }
-}
-
 // Checks to see whether an expression contains comments inline inside of it
 // This can only happen if the expression is a BinOp
 // We should ignore any comments which are trailing for the whole expression, as they are not inline
 pub fn expression_contains_inline_comments(expression: &Expression) -> bool {
     match expression {
         Expression::BinaryOperator { lhs, binop, rhs } => {
-            binop_contains_comments(binop) || expression_contains_comments(lhs)
+            contains_comments(binop) || contains_comments(lhs)
             // Check if the binop chain still continues
             // If so, we should keep checking the expresion
             // Otherwise, stop checking
@@ -851,13 +852,13 @@ pub fn expression_contains_inline_comments(expression: &Expression) -> bool {
                 Expression::BinaryOperator { .. } => expression_contains_inline_comments(rhs),
                 Expression::UnaryOperator { unop, expression } => {
                     let op_contains_comments = match unop {
-                        UnOp::Minus(token) | UnOp::Not(token) | UnOp::Hash(token) => token_contains_comments(token),
+                        UnOp::Minus(token) | UnOp::Not(token) | UnOp::Hash(token) => contains_comments(token),
                         other => panic!("unknown node {:?}", other)
                     };
                     op_contains_comments || expression_contains_inline_comments(expression)
                 }
                 Expression::Value{ .. } => false,
-                Expression::Parentheses { .. } => expression_contains_comments(rhs),
+                Expression::Parentheses { .. } => contains_comments(rhs),
                 other => panic!("unknown node {:?}", other),
             }
         }
